

capture mata mata drop Estimate_Distal_Mean_And_SE()
mata:

void function Estimate_Distal_Mean_And_SE(real matrix Z, real scalar nclassM, real matrix in_weight, 
								real scalar is_identity_link, real scalar is_log_link, real scalar is_logit_link, 
								real matrix linear_predictor, real matrix distal_std_err, real matrix cov_matrix_linear_predictor){
	weightM = in_weight
	//stata(`"dis "weightM = ""')
	//weightM[1..10,]
	//stata(`"dis "Z = ""')
	//Z[1..10]
	N = rows(Z)	
	distal_mean	= J( nclassM, 1, .)		  
	for( c = 1; c <= nclassM; c++) {	
		distal_mean[c] = sum(weightM[,c]:*Z)/sum(weightM[,c])			  
	}
	//stata(`"dis "distal_mean = ""')
	//distal_mean
	if( is_identity_link == 1 ) {
		linear_predictor = distal_mean
	}
	;
	if( is_log_link == 1 ) {
		for( c = 1; c <= nclassM; c++) {			
			if(distal_mean[c,1] < 1e-15 ) {
				distal_mean[c,1] = 1e-15
			}
			;
		}
		linear_predictor = log(distal_mean)
	}
	;
	if( is_logit_link == 1 ) {
		for( c = 1; c <= nclassM; c++) {			
			if(distal_mean[c,1] < 1e-15 ) {
				distal_mean[c,1] = 1e-15
			}
			;
			if(distal_mean[c,1] > .999999999999999  ) {
				distal_mean[c,1] = .999999999999999 
			}
			;
		}
		linear_predictor = log(distal_mean:/(1.0:-distal_mean:+1e-30))
	}	
	;
	//stata(`"dis "linear_predictor""')
	//linear_predictor
	empirical_cov_Z = J(nclassM,nclassM,0)
	for( i = 1; i <= N; i++) {			
		thisSubjectsResiduals = J(nclassM,1,0)
		for( c = 1; c <= nclassM; c++) {
			thisSubjectsResiduals[c,1] = (weightM[i,c])*(Z[i,1]-distal_mean[c,1])	
		}	
		empirical_cov_Z = empirical_cov_Z + thisSubjectsResiduals*thisSubjectsResiduals'			
	}
	//stata(`"dis "empirical_cov_Z""')
	//empirical_cov_Z
	inv_Diag_Sum_Weights = J(nclassM,nclassM,0)
	for( c = 1; c <= nclassM; c++) {
		if(is_identity_link == 1){
			inv_Diag_Sum_Weights[c,c] = 1/sum(weightM[,c])
		}
		;
		if(is_log_link == 1 ){
			inv_Diag_Sum_Weights[c,c] = 1/sum(weightM[,c]:*distal_mean[c,1])
		}
		;
		if(is_logit_link == 1){
			inv_Diag_Sum_Weights[c,c] = 1/sum(weightM[,c]:*(distal_mean[c,1]*(1-distal_mean[c,1])))
		}
		;
	}
	//stata(`"dis "inv_Diag_Sum_Weights""')
	//inv_Diag_Sum_Weights
	cov_matrix_linear_predictor = ((nclassM*N)/(nclassM*N-nclassM))*inv_Diag_Sum_Weights*empirical_cov_Z*inv_Diag_Sum_Weights
	distal_std_err = sqrt(diagonal(cov_matrix_linear_predictor))
}
end

capture mata mata drop Distal_NoGroup_BCH()
mata:
//mata clear
void function Distal_NoGroup_BCH(string scalar distal, string scalar in_gamma, real scalar is_binary, real scalar is_categorical, 
								  real scalar is_continuous, real scalar is_count, real scalar is_naive_adjustment_method, 
								  real scalar is_proportional_assignment, real scalar there_are_sampling_weights, 
								  real scalar in_alpha, | string scalar weight) {
	Z = st_data(.,distal)
	gammaM = st_matrix(in_gamma)
	//stata(`"dis "gammaM = ""')
	//gammaM	
	//beta0M=	st_matrix(in_beta0)	
	//stata(`"dis "beta0M = ""')
	//beta0M		  
	//beta1M=	st_matrix(in_beta1)	
	//stata(`"dis "beta1M = ""')
	//beta1M		  	  

	critValue = invnormal(1-0.5*in_alpha)

	nclassM = cols(gammaM)
	
	if(there_are_sampling_weights == 1 ){
		sampling_weight= st_data(., weight)
	}
	;
	for( c=1; c<=nclassM; c++){
		tmp = st_data(., "_post_prob" + strofreal(c))
		if( c == 1) {
			postprobs = tmp
		} 
		else {
			postprobs = postprobs, tmp
		}
	}
	N = rows(postprobs)
	if(is_proportional_assignment == 1 ){
		unadjusted_Weights= postprobs
	} 
	else {
		modal = J(N,1,.)
		for( i=1; i<=N; i++){
			u = J(0,1,.)
			w = J(0,2,.)
			maxindex(postprobs[i,],1,u,w )
			modal[i,1] = u[1]  
			//modal[i,1] = select((1..rows(postprobs)), (postprobs[i,]:==max(postprobs[i,])))
		}
		unadjusted_Weights = J(N,nclassM,.)
		for( i=1; i<=N; i++){
			for( c=1; c<=nclassM; c++){
				unadjusted_Weights[i,c] = (modal[i,1] == c)
			}
		}
	}
	if(there_are_sampling_weights == 1) { 
		for( c=1; c<=nclassM; c++){
			unadjusted_Weights[,c] = unadjusted_Weights[,c] :* sampling_weight
		}
	}
	;
	//stata(`"dis "unadjusted_Weights = ""')
	//unadjusted_Weights[1..10,]
    D = J(nclassM,nclassM,.)
    for( t = 1; t <= nclassM; t++){
		for( s = 1; s <= nclassM; s++) {
			D[t,s] = sum(unadjusted_Weights[,s]:*postprobs[,t])/(N*gammaM[t])
		}
	}
    Dinv = luinv(D)
    BCH_Weights = unadjusted_Weights*Dinv
	if(is_naive_adjustment_method == 1 ){
		chosen_Weights= unadjusted_Weights
	} 
	else {
		chosen_Weights = BCH_Weights
	}	
	//stata(`"dis "chosen_Weights""')
	//chosen_Weights[1..10,]	
	if(is_continuous==1 | is_count==1 ){
		Classes = (1::nclassM)
        distal_mean = J(nclassM,1,.)
        distal_std_error_for_mean = J(nclassM,1,.)
        cov_matrix_linear_predictor = J(nclassM,nclassM,.)
		Z = st_data(.,distal)
		Estimate_Distal_Mean_And_SE(Z, /* input */
									nclassM, /* input */
									chosen_Weights, /* input */
									1, 0, 0, /* treat as identity link for purposes of getting 
                                                 direct estimates and symmetric standard errors 
                                                 for the expected values; this is statistically
                                                 inefficient but asymptotically valid and roughly
                                                 follows the approach of Latent GOLD. */
									distal_mean, /* output */
									distal_std_error_for_mean, /* output */
									cov_matrix_linear_predictor)
		if(is_count==1 ) {
			if(sum(distal_mean :< 0 )>0){
				(distal_mean :> 0) :* distal_mean
				for(c=1; c<=nclassM; c++){
					if(distal_std_error_for_mean[c,1] <0){
						distal_std_error_for_mean[c,1] = .
					}
					;
				}
            }
			;
        }
		; 
	    distal_CI_Upper = distal_mean :+ critValue:*distal_std_error_for_mean
        distal_CI_Lower = distal_mean :- critValue:*distal_std_error_for_mean
    	estimatesM = J(0,5,.) \ (Classes,distal_mean,distal_std_error_for_mean,distal_CI_Lower,distal_CI_Upper)
					           
	} 
	else if(is_binary==1){
		Classes = (1::nclassM)
 		distal_prob = J(nclassM,1,.)
        distal_std_error_for_prob = J(nclassM,1,.)
        cov_matrix_distal_probs= J(nclassM,nclassM,.)
		Z = st_data(.,distal)
        Estimate_Distal_Mean_And_SE( Z, /* input */
									 nclassM,  /* input */
                                     chosen_Weights,  /* input */
                                     1, 0, 0, /* treat as identity link for purposes of getting 
                                                 direct estimates and symmetric standard errors 
                                                 for the expected values; this is statistically
                                                 inefficient but asymptotically valid and roughly
                                                 follows the approach of Latent GOLD. */
                                     distal_prob,  /* output */
                                     distal_std_error_for_prob, /* output */
                                     cov_matrix_distal_probs)
		if(sum(distal_prob :< 0 )>0){
			distal_prob = (distal_prob :> 0) :* distal_prob
			for(c=1; c<=nclassM; c++){
				if(distal_std_error_for_prob[c,1] <0){
					distal_std_error_for_prob[c,1] = .
				}
				;
			}
		}
		;
		if(sum(distal_prob :> 1 )>0){
			for(c=1; c<=nclassM; c++){
				if(distal_prob[c,1] > 1){
					distal_prob[c,1] = 1
					distal_std_error_for_prob[c,1] = .
				}
				;
			}
		}
		;
	    distal_CI_Upper = distal_prob :+ critValue:*distal_std_error_for_prob
        distal_CI_Lower = distal_prob :- critValue:*distal_std_error_for_prob
		estimatesM = J(0,5,.) \ (Classes,distal_prob,distal_std_error_for_prob,distal_CI_Lower,distal_CI_Upper)
		
		distal_probability = J(nclassM,1,.) 
        distal_log_odds = J(nclassM,1,.) 
        distal_std_error_for_log_odds = J(nclassM,1,.)
        cov_matrix_log_odds = J(nclassM,nclassM,.) 
		Z = st_data(.,distal)
		Estimate_Distal_Mean_And_SE(Z, /* input */
									nclassM,  /* input */
                                    chosen_Weights,  /* input */
                                    0, 0, 1, /* now correctly treat as binary. */
                                    distal_log_odds,  /* output */
                                    distal_std_error_for_log_odds, /* output */
                                    cov_matrix_log_odds)
        distal_probability = exp(distal_log_odds):/(1 :+ exp(distal_log_odds))
        CI_lower_log_odds = distal_log_odds :- critValue :* distal_std_error_for_log_odds
        CI_upper_log_odds = distal_log_odds :+ critValue :* distal_std_error_for_log_odds
        CI_lower_probability = exp(CI_lower_log_odds):/(1 :+ exp(CI_lower_log_odds))
        CI_upper_probability = exp(CI_upper_log_odds):/(1 :+ exp(CI_upper_log_odds))	

		log_oddsM = J(0,7,.) \ (Classes,distal_probability,distal_log_odds,CI_lower_log_odds,CI_upper_log_odds,CI_lower_probability,CI_upper_probability)
		st_matrix("log_oddsMatrix", log_oddsM)	
		
	} 
	else if(is_categorical==1){
		/* This case is different from the others because we have to loop through
		   the categories and get estimates for each category. */
		category_min = colmin(Z)
		category_max = colmax(Z)
		num_cats = category_max - category_min + 1

	   /* The minimum to maximum category numbers.  They should usually be 
		  1 through the number of categories, but we might have to deal with the 
		  possibility that they might not be. */
        outcome_category  =  colshape((category_min::category_max) * J(1,nclassM,1), 1)
		Classes = colshape(J(num_cats,1,1) * (1..nclassM),1)
        distal_prob = J(rows(outcome_category),1,.)
        distal_std_error_for_prob = J(rows(outcome_category),1,.)
		for(this_Category = category_min; this_Category<= category_max; this_Category++){
			Z_This_Category = J(N, 1, 1)
			Z_This_Category = (Z :== this_Category) 
			/*for(i=1; i<=N; i++){
				Z_This_Category[i,1] = ( Z[i,1]==This_Category )
			}*/
			
			distal_prob_this_category = J(nclassM,1,.)
			distal_std_error_this_category = J(nclassM,1,.)
			cov_matrix_linear_pred_this_cat = J(nclassM,nclassM,.)
			//stata(`"dis "Z_This_Category""')
			//Z_This_Category[1::20]
            Estimate_Distal_Mean_And_SE( Z_This_Category,
										 nclassM,  /* input */
                                         chosen_Weights,  /* input */
                                         1, 0, 0, /* treat as identity link for purposes of getting 
													a direct estimate for the proportion and symmetric 
													standard errors for it */
                                         distal_prob_this_category,  /* output */
                                         distal_std_error_this_category, /* output */
                                         cov_matrix_linear_pred_this_cat)
		   //stata(`"dis "distal_prob_this_category""')
		   //distal_prob_this_category
		   //stata(`"dis "distal_std_error_this_category""')
		   //distal_std_error_this_category
		   //stata(`"dis "cov_matrix_linear_pred_this_cat""')
		   //cov_matrix_linear_pred_this_cat
		   distal_prob[select((1::rows(outcome_category)), rowmax(outcome_category :== this_Category))] = distal_prob_this_category
           distal_std_error_for_prob[select((1::rows(outcome_category)), rowmax(outcome_category :== this_Category))] = distal_std_error_this_category 
		}
		if(sum(distal_prob :< 0 )>0){
			for(c=1; c<=nclassM; c++){
				if(distal_prob[c,1] <0){
					distal_prob[c,1] = .
					distal_std_error_for_prob[c,1] = .
				}
				;
			}
		}
		;
		if(sum(distal_prob :> 1 )>0){
			for(c=1; c<=nclassM; c++){
				if(distal_prob[c,1] > 1){
					distal_prob[c,1] = .
					distal_std_error_for_prob[c,1] = .
				}
				;
			}
		}
		;
		estimatesM = J(0,4,.) \ (Classes,outcome_category, distal_prob,distal_std_error_for_prob) 
	}
	if(is_continuous==1 | is_count==1 | is_binary==1){
		linear_predictor = J(nclassM,1,.)
        distal_std_error = J(nclassM,1,.)
        cov_matrix_linear_predictor = J(nclassM,nclassM,.)
		Estimate_Distal_Mean_And_SE(Z,  /* input */
									nclassM,  /* input */
									chosen_Weights,  /* input */
									is_continuous, /* input */   /* to use identity link */
									is_count, /* input */    /* to use log link */
									is_binary, /* input */   /* to use logit link */
									linear_predictor,  /* output */
									distal_std_error, /* output */
									cov_matrix_linear_predictor) /* output */     
		/* Now we have the correct covariance matrix for the linear predictor vector, which is 
	   defined here as the link function applied to each the distal means.  This covariance
	   matrix is stored in Cov_Matrix_Linear_Predictor.  We will use it to do significance 
	   tests on the differences between linear predictors (transformed distal means) for 
	   each class.  This can be done using matrix algebra.  
	   Consider the continuous case for simplicity so that the distal mean
	   and the linear predictor are the same thing (identity link).  Then the estimates for
	   the distal mean parameters are a vector,  say maybe [4.2, 1.9, 3.8] for classes 1, 2, 
	   and 3, stored in Linear_Predictor, and their covariance matrix is in 
	   Cov_Matrix_Linear_Predictor.  We can calculate a linear contrast by taking the inner 
	   product of the mean vector with a vector L; for instance, 
	   E = L` * mu = [-1, +1, 0]` * [4.2, 1.9, 3.8] = 1.9 - 4.2, the difference between classes 2 and 1.
	   (The ` sign is SAS symbol for transpose.) 
	   The variance of this contrast E is the quadratic form L`*Cov_Matrix_Linear_Predictor*L.
	   The Wald statistic is the square of E, divided by its variance.  It has 1 degree of 
	   freedom if 1 parameter is being compared to 1 other parameter.  We get the p-value for 
	   the Wald statistic by comparing it to a chi-squared distribution.			*/								   
		/* Calculate specific contrasts */
		contrasts = J(nclassM*(nclassM-1)/2,nclassM,0)
		namesM = J(nclassM*(nclassM-1)/2,1,0)
		firstClassNames = J(nclassM*(nclassM-1)/2,1,0)  
		secondClassNames = J(nclassM*(nclassM-1)/2,1,0)    
		estimate = J(nclassM*(nclassM-1)/2,1,0)
		std_error = J(nclassM*(nclassM-1)/2,1,0)
		wald_statistic = J(nclassM*(nclassM-1)/2,1,0)
		df = J(nclassM*(nclassM-1)/2,1,0)
		p_value = J(nclassM*(nclassM-1)/2,1,0)
		row = 0
        for(firstClass = 1; firstClass <=(nclassM-1); firstClass++){
			for(secondClass = (firstClass+1); secondClass<=nclassM; secondClass++){
				row = row + 1
				contrasts[row,firstClass] = -1
				contrasts[row,secondClass] = 1 
				firstClassNames[row] = firstClass
				secondClassNames[row] = secondClass
			}
        }
		if(is_continuous == 1){
			Name = "Diff_in_Means"
		}
		;
		if(is_count == 1){
			Name = "Diff_in_Log_Means"
		}
		;
		if(is_binary == 1){
			Name = "Diff_in_Log_Odds"
		}
		;
		namesM = Name :+":Class_" :+strofreal(secondClassNames) :+"_vs_":+ strofreal(firstClassNames)	
	    for(row = 1; row<= rows(contrasts); row++){ 
			estimate[row] =  contrasts[row,]*linear_predictor
			std_error[row] = sqrt(contrasts[row,]*cov_matrix_linear_predictor*contrasts[row,]')
			   /* square root of the variance */
			wald_statistic[row] = (estimate[row]/std_error[row])^2
			df[row] = 1
			p_value[row] = 1-chi2(1,wald_statistic[row])
		}							
		/* Calculate omnibus test.  "Omnibus" means the test of whether all of the means 
		are equal.  We don't care about estimates here, only a test.  That is because we're 
		not asking about the difference between class A and class B, but the differences
		between all the classes.  Instead of a vector L, we have a matrix L containing 
		the largest number of rows that would not lead to a singular L * Cov_Matrix_Linear_Predictor * L`,
		for example,
		[-1, 1, 0,
		 -1, 0, 1] 
		 for 3 classes. 			*/
		omnibus_df = nclassM-1
		rows = (1::omnibus_df)
		omnibus_estimate_vector = contrasts[rows,]*linear_predictor
		omnibus_error_matrix = contrasts[rows,]*cov_matrix_linear_predictor*contrasts[rows,]'
		omnibus_wald_statistic = omnibus_estimate_vector' * invsym(omnibus_error_matrix) * omnibus_estimate_vector
		omnibus_p_value = 1-chi2(omnibus_df,omnibus_wald_statistic)
		namesM = namesM \ "Omnibus_Test"
		estimate = estimate \ .
		std_error = std_error \ .
		wald_statistic = wald_statistic \ omnibus_wald_statistic
		df = df \ omnibus_df
		p_value = p_value \ omnibus_p_value

		testsM = J(0,5,.) \ (estimate, std_error, wald_statistic, df, p_value)
		
	} 
	else if(is_categorical==1){
		/* I had to write the code for categorical outcomes separate because it uses a different formula.
		I adopted this and the other formulas from the sandwich formula used in the documentation of 
		SAS PROC SURVEYLOGISTIC, even though we do not call this PROC.  */
		//total_omnibus_df = 0
		//total_omnibus_wald_statistic = 0
		
		min_cat = colmin(Z) /* smallest category number;  most users will use 1 here */
		max_cat = colmax(Z) /* largest category number */
		num_cats = max_cat - min_cat + 1
		cats = (min_cat :: max_cat)   //(min_cat..max_cat)
		G = J(nclassM*(num_cats-1),nclassM*(num_cats-1),0)
		for( i = 1; i<= N; i++){
			e_i = J(nclassM*(num_cats-1),1,.)
			row = 0
			for( c = 1; c<= nclassM; c++){
				for( this_category_index = 1; this_category_index <= (num_cats-1); this_category_index++){
					row = row + 1					
					this_category = cats[this_category_index]
					Z_i_this_category = (Z[i]:==this_category)
					distal_prob_this_category = sum(chosen_Weights[,c]:*(Z:==this_category))/sum(chosen_Weights[,c])
					e_i[row] = (chosen_Weights[i,c])* (Z_i_this_category - distal_prob_this_category)
				}
			}
			G = G :+ e_i * e_i'
		}
		Q = J(nclassM*(num_cats-1),nclassM*(num_cats-1),0)
		for(c = 1; c<= nclassM; c++){
			pi_vector_this_class = J(num_cats-1,1,.)
			diag_pi_vector_this_class = J(num_cats-1,num_cats-1,0)
			for(this_category_index = 1;  this_category_index <= (num_cats-1); this_category_index++){ 
				this_category = cats[this_category_index]
				z_this_category = (Z:==this_category)
				distal_prob_this_class_and_cat = sum(chosen_Weights[,c]:*z_this_category)/sum(chosen_Weights[,c])
				pi_vector_this_class[this_category_index] = distal_prob_this_class_and_cat
				diag_pi_vector_this_class[this_category_index,this_category_index] = distal_prob_this_class_and_cat
			}
			for(i = 1; i<= N; i++){
				new_block_contrib = chosen_Weights[i,c]*(diag_pi_vector_this_class - pi_vector_this_class*pi_vector_this_class')
				places = ((c-1)*(num_cats-1)+1)::(c*(num_cats-1))
				Q[places,places] = Q[places,places] + new_block_contrib
			}
		} 
		/* Q is now the model-based covariance matrix for the distal linear predictors. */
		Qinv = invsym(Q)
		cov_matrix_linear_predictor = ((nclassM*N-1)/(nclassM*N-nclassM)):*(Qinv*G*Qinv)
					 /* sandwich formula */			
		linear_predictor = J(nclassM*(num_cats-1),1,.)
		row = 0
		for(c = 1; c<= nclassM; c++){
			for( this_category_index = 1;  this_category_index <=(num_cats-1); this_category_index++){
				row = row + 1				
				this_category = cats[this_category_index]
				Z_this_category = (Z:==this_category)
				Z_baseline = (Z:==max_cat)
				distal_prob_this_category = sum(chosen_Weights[,c]:*Z_this_category)/sum(chosen_Weights[,c])
				distal_prob_baseline = sum(chosen_Weights[,c]:*Z_baseline)/sum(chosen_Weights[,c])
				linear_predictor[row] = log(distal_prob_this_category+1E-20)-log(distal_prob_baseline+1e-20)
			}
		}
		/* Calculate omnibus contrasts.  I decided not to calculate estimates in the 
		categorical case, but only do the omnibus test.  Otherwise we would need to 
		provide a test for each pair of classes within each response category, which 
		feels like too many null hypotheses to me. */
		contrasts = J((nclassM-1)*(num_cats-1),nclassM*(num_cats-1),0)
		row = 0
		for(c = 2; c<= nclassM; c++){
			for(this_category_index = 1;  this_category_index<= (num_cats-1); this_category_index++){
				row = row + 1				
				contrasts[row,this_category_index] = 1
				contrasts[row,(c-1)*(num_cats-1)+this_category_index] = -1
			}
		}
		omnibus_estimate_vector = contrasts*linear_predictor
		omnibus_error_matrix = contrasts*cov_matrix_linear_predictor*contrasts'
			  /* the symbol ` means transpose */
		wald_statistic = omnibus_estimate_vector' * invsym(omnibus_error_matrix) * omnibus_estimate_vector
		Name = "Omnibus_Test";  
		df = (nclassM-1) * (num_cats-1)
		p_value = 1-chi2(df,wald_statistic)
		if(sum(distal_prob :== . ) >0){
			wald_statistic = .
			p_value = .
		}
		;
		namesM = Name
		testsM = J(0,3,.) \ (wald_statistic, df, p_value) 
		//testsM
		
	}
	if(cols(estimatesM)== 5){
		estimatesM = sort(estimatesM,1)	 
	}
	else if(cols(estimatesM)== 4){
		estimatesM = sort(estimatesM,(1,2))
	}
	
	
	st_matrix("estimatesMatrix", estimatesM)
	st_local("testColNames", invtokens(namesM'))
	st_matrix("testsMatrix", testsM)
}
end
	
capture program drop doLCA_Distal_BCH
program doLCA_Distal_BCH, rclass
	version 11.0
	/* metric: options are binary/logistic
		                   or categorial/nominal
						   or count/poisson
				           or continuous/normal/numerical
	  adjustment_method: options are unadjusted/naive
					                 or else
	  assignment:	proportional 
	                or else  								*/
	
	syntax varname,						///
		gammaList(numlist)				///
		metric(string)             		///
		[adjustment_method(string)       ///
		assignment(string)             	///
		GROUPs(varlist)      			///
		 alpha(real 0.05)  				///
		 weight(varlist)]                

	 drop if missing(`varlist')   //remove cases with missing distal data
	 //dis "gammaList" `gammaList'
	 //dis "betaList" `betaList'
 	 tempname is_binary is_categorical is_count is_continuous is_naive_adjustment_method is_proportional_assignment ///
			  is_proportional_assignment there_are_groups hasWeight nclass ngroup ///
			  gammaMatrix gamma_eachg betaMatrix beta0 beta1 safeM gammagiven0 gammagiven1 ///
			  unadjusted_weights distal_mean groupNum ///
			  estimates_all log_odds_all distal_tests_all
			  			  		  
	 scalar `is_binary' = 0
	 scalar `is_categorical' = 0
	 scalar `is_count' = 0
	 scalar `is_continuous' = 0
	 scalar `is_naive_adjustment_method' = 0
	 scalar `is_proportional_assignment' = 0
	 scalar `hasWeight' = 1
	 scalar `there_are_groups' = 0
	 if(upper(trim(`"`metric'"')) == "BINARY" | upper(trim(`"`metric'"')) == "LOGISTIC") {
		scalar `is_binary' = 1
		//qui sum `varlist'
		//assert `varlist' == r(max) | `varlist' == r(min)
		if( `varlist' != 0 & `varlist' != 1 ) {
			display as error "Error: Please code a binary distal outcome as 0 for no and 1 for yes."
			exit 198
		}
	 }		  
	 if(upper(trim(`"`metric'"')) == "CATEGORICAL" | upper(trim(`"`metric'"')) == "NOMINAL") {
		scalar `is_categorical' = 1
	 }		  
	 if(upper(trim(`"`metric'"')) == "COUNT" | upper(trim(`"`metric'"')) == "POISSON") {
		scalar `is_count' = 1
	 }		  
	 if(upper(trim(`"`metric'"')) == "CONTINOUS" | upper(trim(`"`metric'"')) == "NORMAL" | upper(trim(`"`metric'"')) == "NUMERICAL") {
		scalar `is_continuous' = 1
	 }	
	 if(`is_binary'+`is_categorical'+`is_count'+`is_continuous' != 1){
		display as error "Error :  Could not recognize option specified for metric."
		exit 198
	 }	  
	 if(upper(trim(`"`adjustment_method'"')) == "UNADJUSTED" | upper(trim(`"`adjustment_method'"')) == "NAIVE") {
		scalar `is_naive_adjustment_method' = 1
	 }		  
	 if(upper(trim(`"`assignment'"')) == "PROPORTIONAL" ) {
		scalar `is_proportional_assignment' = 1
	 }	
	 	 	  
 	 if(`"`groups'"'!="") {
		scalar `there_are_groups' = 1
	 }		  
	 if(`is_count' == 1 | `is_continuous' == 1){
		dis "BCH Estimation of Means of `varlist' by Latent Class"
	 }
	 else
	 {
		dis "BCH Estimation of Proportions of `varlist' by Latent Class"
	 }
	 if ( `"`weight'"' == "") {
		scalar `hasWeight' = 0
	 }
	 
	scalar `ngroup' = 1
    if ( `"`groups'"' != "") {
		 qui sum `groups'
		 scalar `ngroup' = r(max)
	}
	 
	local numList : word count `gammaList'
	scalar `nclass' = `numList'/`ngroup'
	//dis `nclass'
	
	matrix define `gammaMatrix'=J(`ngroup',`nclass',0.0)
	forvalues i=1/`=`ngroup'' {
		forvalues j=1/`=`nclass'' {
		    local k= (`i'-1)*`nclass' +`j'
			local value : word `k' of `gammaList'
			mat `gammaMatrix'[`i',`j']=`value'
		}
	}
	//mat list `gammaMatrix'
 	
 	local N=_N
	matrix define `distal_mean'=J(`nclass',1,.)
	set matsize `N'
	matrix define `unadjusted_weights'=J(`N',`nclass',.)
    local rownamesForTestsAll ""
	//list in 1/10
	if(`there_are_groups' == 1){
		forvalues i = 1(1) `=`ngroup'' {
			preserve
			keep if `groups' ==`i'
			mat `gamma_eachg' = `gammaMatrix'[`i',1...]
			//mat list `gamma_eachg'
			
			//dis "For Group `i' of `ngroup' on Variable `groups'"
			mata: Distal_NoGroup_BCH("`varlist'", "`gamma_eachg'", `=`is_binary'', `=`is_categorical'', `=`is_continuous'', `=`is_count'', `=`is_naive_adjustment_method'', `=`is_proportional_assignment'',`=`hasWeight'',`alpha', "`weight'")
			if( `i' == 1 ){
				mat `estimates_all' = (J(rowsof(estimatesMatrix),1,`i'),estimatesMatrix)
			} 
			else {
				mat `estimates_all' = (`estimates_all' \ J(rowsof(estimatesMatrix),1,`i'),estimatesMatrix)
			}

			if(`is_binary' == 1) {
				if( `i' == 1 ){
					mat `log_odds_all' = (J(rowsof(log_oddsMatrix),1,`i'),log_oddsMatrix)
				} 
				else {
					mat `log_odds_all' = (`log_odds_all' \ J(rowsof(log_oddsMatrix),1,`i'),log_oddsMatrix)
				}
				
			}
			
			if( `i' == 1 ){
				mat `distal_tests_all' = (J(rowsof(testsMatrix),1,`i'),testsMatrix)
			} 
			else {
				mat `distal_tests_all' = (`distal_tests_all' \ J(rowsof(testsMatrix),1,`i'),testsMatrix)
			}
			
			if( `i' == 1 ){
				local rownamesForTestsAll `"`testColNames'"'
			} 
			else {
				local rownamesForTestsAll `"`rownamesForTestsAll' `testColNames'"'
			}

			restore
		}
	}
	else {
		mata: Distal_NoGroup_BCH("`varlist'", "`gammaMatrix'", `=`is_binary'', `=`is_categorical'', `=`is_continuous'', `=`is_count'', `=`is_naive_adjustment_method'', `=`is_proportional_assignment'', `=`hasWeight'',`alpha', "`weight'")
		mat `estimates_all' = estimatesMatrix
		
		if(`is_binary' == 1) {
			mat `log_odds_all' = log_oddsMatrix
		}
		
		local rownamesForTestsAll `"`testColNames'"'
		mat `distal_tests_all' = testsMatrix
			
	}	
	
	if(`there_are_groups' == 1){
		if(colsof(`estimates_all') == 5){
			local estimates_all_ColNames "GROUP ClASS CATEGORY DIST_PROB DIST_STD_ERROR"  
		} 
		else if(colsof(`estimates_all') == 6){
			local estimates_all_ColNames "GROUP ClASS DISTAL_PROB DIST_STD_ERROR Dist_CI_Lo Dist_CI_Up"  
		}

		if(`is_binary' == 1) {
			local log_odds_all_ColNames "GROUP ClASS DISTAL_PROB DIST_LOG_ODDS CI_LOW_LOG_ODDS CI_UPP_LOG_ODDS CI_LOW_PROB CI_UPP_PROB"  
		}
		
		if(colsof(`distal_tests_all') == 6){
			local distal_tests_all_ColNames "GROUP ESTIMATE STD_ERROR WALD_STATISTICS DF P_VALUE"
		}
		else if(colsof(`distal_tests_all') == 4){
			local distal_tests_all_ColNames "GROUP WALD_STATISTICS DF P_VALUE"
		}
	} 
	else {
		if(colsof(`estimates_all') == 4){
			local estimates_all_ColNames "ClASS CATEGORY DISTAL_PROB DIST_STD_ERROR"  
		} 
		else if(colsof(`estimates_all') == 5){
			local estimates_all_ColNames "ClASS DISTAL_PROB DIST_STD_ERROR Dist_CI_Low Dist_CI_Upp"  
		}

		if(`is_binary' == 1) {
			local log_odds_all_ColNames "ClASS DISTAL_PROB DIST_LOG_ODDS CI_LOW_LOG_ODDS CI_UPP_LOG_ODDS CI_LOW_PROB CI_UPP_PROB"  
		}
		
		if(colsof(`distal_tests_all') == 5){
			local distal_tests_all_ColNames "ESTIMATE STD_ERROR WALD_STATISTICS DF P_VALUE"
		}
		else if(colsof(`distal_tests_all') == 3){
			local distal_tests_all_ColNames "WALD_STATISTICS DF P_VALUE"
		}
	}
	
	matrix colnames `estimates_all' = `estimates_all_ColNames' 
	if(`is_binary' == 1) {
		matrix colnames `log_odds_all' = `log_odds_all_ColNames' 			
	}
	matrix colnames `distal_tests_all' = `distal_tests_all_ColNames' 
	
	local matLabel ""
	if(`is_proportional_assignment' == 1) {
		if(`is_naive_adjustment_method' == 1){
			local matLabel "Estimates using Unadjusted Proportional Weighting" 
		} 
		else {
			local matLabel  "Estimates using BCH Proportional Weighting" 
		}
	} 
	else {
		if(`is_naive_adjustment_method' == 1) {
			local matLabel  "Estimates using Unadjusted Modal Weighting"
		} 
		else {
			local matLabel  "Estimates using BCH Modal Weighting"
		}
	}
	mat list `estimates_all', title("`matLabel'") //twidth(20)

	
	if(`is_binary' == 1) {
		mat list `log_odds_all', title("Confidence Intervals for Probabilities")   //twidth(20)	
	}
	
	
	//dis "`rownamesForTestsAll'"
	mat rownames `distal_tests_all' = `rownamesForTestsAll'
	mat list `distal_tests_all', title("Wald Chi-Squared Tests")  //twidth(20)

	return matrix distal_estimates `estimates_all'
	if(`is_binary' == 1 ){
		return matrix distal_log_odds `log_odds_all'
	}
	return matrix distal_tests `distal_tests_all'
end 


